######################################################################
## This file is generated by scikitplot's build process
## It contains CITATION.bib contents at the time of building this package.
######################################################################
import pathlib

# from scikitplot import sp_logging as logging
from scikitplot import sp_logger as logging

_built_with_meson = True

__all__ = [
  "CITATION",
  "_get_bibtex",
  "__bibtex__",
  "__citation__",
]

######################################################################
## Create CONFIG by CITATION.bib contents
######################################################################

CITATION = """% --------------------------------------------------------------------
% CITATION.bib file for
% This file provides citation information for users
% who want to cite the library, related papers, and books.
% --------------------------------------------------------------------

@software{scikit-plots:vlatest,
  title   = {{ scikit-plots: Machine Learning Visualization in Python }},
  author  = { scikit-plots developers },
  month   = { 11 },
  year    = { 2024 },
  doi     = { 10.5281/zenodo.13367000 },
  url     = { https://scikit-plots.github.io },
  version = { latest },
  message = { Scikit-plot is the result of an unartistic data scientist's dreadful realization that visualization is one of the most crucial components in the data science process, not just a mere afterthought. },
}"""

# Set the bibtex entry to the article referenced in CITATION.
def _get_bibtex(
  preferred_type:str = 'software',
  ):
    _bibtex = ''
    try:
        # Attempt to read the CITATION.bib file
        refs = (pathlib.Path(__file__).parent / "../CITATION.bib").read_text().split(f"@{preferred_type}")[1:][0]
        _bibtex = f"@{preferred_type}{refs}" if refs else ""
        logging.debug("_bibtex from file CITATION.bib")
    except FileNotFoundError:
        # If CITATION.bib file is not found,
        # fall back to the defined CITATION text string
        refs = CITATION.split(f"@{preferred_type}")[1:][0]
        _bibtex = f"@{preferred_type}{refs}" if refs else ""
        logging.debug("_bibtex from attr CITATION text")
    except Exception as e:
        # Handle any unexpected errors
        logging.error(f"Unexpected error while reading CITATION.bib: {e}")
    return _bibtex

__bibtex__ = __citation__ = _get_bibtex()

######################################################################
## ...
######################################################################
